@@ -1,10 +1,11 @@
 package Module::Path;
 # ABSTRACT: get the full path to a locally installed module
-$Module::Path::VERSION = '0.11';
+$Module::Path::VERSION = '0.12';
 use 5.006;
 use strict;
 use warnings;
 use File::Basename 'dirname';
+use Cwd qw/ abs_path /;
 
 require Exporter;
 
@@ -32,20 +33,21 @@ sub module_path
     ($relpath = $module) =~ s/::/$SEPARATOR/g;
     $relpath .= '.pm' unless $relpath =~ m!\.pm$!;
 
+    DIRECTORY:
     foreach my $dir (@INC) {
+        next DIRECTORY if not defined($dir);
+
         # see 'perldoc -f require' on why you might find
         # a reference in @INC
-        next if ref($dir);
+        next DIRECTORY if ref($dir);
 
-        # If $dir is a symlink, then we resolve it.
-        # Returing a full path containing a symlinked directory can
-        # cause problems: https://github.com/neilbowers/Module-Path/issues/4
-        if (-l $dir) {
-            my $linkdir = readlink($dir);
-            $dir = index($linkdir, $SEPARATOR) == 0
-                   ? $linkdir
-                   : dirname($dir).$SEPARATOR.$linkdir;
-        }
+        # The directory path might have a symlink somewhere in it,
+        # so we get an absolute path (ie resolve any symlinks).
+        # The previous attempt at this only dealt with the case
+        # where the final directory in the path was a symlink,
+        # now we're trying to deal with symlinks anywhere in the path.
+        $dir = abs_path($dir);
+        next DIRECTORY unless defined($dir);
 
         $fullpath = $dir.$SEPARATOR.$relpath;
         return $fullpath if -f $fullpath;
